package de.hdm_stuttgart.mi.sd1.task2;

/**
 * <p>Representing square polynomials having coefficients of type <code>int</code>.</p>
 *
 * <p>A square polynomial \( p(x)  = a x^2 + b x + c \) is being defined by its three coefficients
 * \(a\), \(b\) and \(c\). We assume all three to be of type <code>int</code>.</p>
 *
 */
public class SquarePolynom {
    private int a, b, c;

    /**
     * <p>Defining a polynomial \( p(x) = a x^2 + b x + c  \).</p>
     *
     * @param a Square coefficient.
     * @param b Linear coefficient.
     * @param c Constant coefficient.
     */
    public SquarePolynom(int a, int b, int c) {
        setA(a);
        setB(b);
        setC(c);
    }

    /**
     * <p>The polynomial's value for a given argument of x.</p>
     *
     * @param x The argument i.e. x = 3.0
     * @return Returning e.g. \( a \cdot 3.0^2 + b \cdot 3.0 + c\) for \( x = 3.0 \).
     */
     public double get(double x) {
        return x * (a * x + b) + c; // Horner's scheme
    }

    /**
     * <p>Re- setting the square coefficient in \( a x² \).</p>
     *
     * @param a The desired new value.
     */
    public void setA(final int a) {
        this.a = a;
    }

    /**
     * <p>Re- setting the linear coefficient in \( b x \).</p>
     *
     * @param b The desired new value.
     */
    public void setB(final int b) {
        this.b = b;
    }

    /**
     * <p>Re- setting the constant coefficient \( c \).</p>
     *
     * @param c The desired new value.
     */
    public void setC(final int c) {
        this.c = c;
    }

    /**
     * <p>Providing zeroes (German: "Nullstellen") of a given square polynomial.</p>
     *
     * @return The ordered finite set of zeroes. In case of an infinite number of zeroes an {@link ArithmeticException}
     *         is being thrown.
     *
     * @throws ArithmeticException In case of \(a = b = c = 0 \) an infinite number of zeroes exists which cannot be
     *                             represented by a finite array of double values.
     */
    public double[] getZeroes() throws ArithmeticException {

        if (0 == a) { // 0 == a, linear polynomial
            if (b != 0) {
                return new double[]{c * (-1.) / b}; //
            } else if (c != 0) {
                return new double[]{}; // No zero exists
            } else {
                throw new ArithmeticException("Polynomial is identical to zero function");
            }
        } else {  // a != 0, square polynomial
            final long radicand = ((long) b) * b - 4L * a * c; // Avoiding overflow errors
            // Caution! final int radicand = ... may fail due to integer overflow

            if (0 > radicand) { // No zero of real (only complex ones)
                return new double[]{};
            } else if (0 == radicand) {
                return new double[]{-b / 2. / a}; // Exactly one zero
            } else { // Two zeroes
                final double radicandRoot = Math.sqrt(radicand),
                x1 = (-b - radicandRoot) / 2 / a,
                x2 = (-b + radicandRoot) / 2 / a;

                if (0 < a) { // Ordering of zeroes by size
                    return new double[]{x1, x2};
                } else {
                    return new double[]{x2, x1};
                }
            }
        }
    }
}